//+------------------------------------------------------------------------
//
//  Copyright @ Baris TAZE
//
//  File:     Generator.h
//
//  Content:  Interface and Implementation for CGenerator class
//  
//  Classes:  CGenerator
//            
//  History:  27-May-2002    created by baris taze .  
//
//-------------------------------------------------------------------------

#if !defined(__GENERATOR_H__)
#define      __GENERATOR_H__

#define DEFAULT_POPULATION_SIZE		1000
#define DEFAULT_CROSS_OVER_RATE		0.6f
#define DEFAULT_MUTATION_RATE		0.001f

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////

class CGenerator  
{
public:	
	CGenerator();
	~CGenerator();

	void Free();

	int   GetPopulationSize( );	
	TPOP* GetCurrentPopulation();

	double GetCrossOverRate( );
	void  SetCrossOverRate( double fCrossOverRate = DEFAULT_CROSS_OVER_RATE );
	
	double GetMutationRate( );
	void  SetMutationRate( double fMutationRate = DEFAULT_MUTATION_RATE );
		
	void  ProduceNextGeneration();
	
	int	  GetIterationCount();	
	double GetCurrentBias ( );
	double GetAveragePerformance( bool bReevaluate = false );
	double GetFittestPerformance( bool bReevaluate = false );
	
	TINDIV* GetFittest();
	TINDIV* GetHistoricalBest();

	void  SetAppData( DWORD_PTR adwAppData );
	DWORD_PTR GetAppData( );

	CTimeSpan GetTotalTimeSpan();	
	int		  GetSuccessiveUnRefinedIterationCount();
	int		  GetLastRefinedIterationNo();

protected:	
	
	void CreatePopulation( int nIndividualCount, int nBitSize );
	bool RestoreHistoricalBest();

	virtual void PerformMutation( TINDIV*& rpPerson );
	virtual void PerformCrossOver( TINDIV*& rpPerson, TINDIV*& rpPair );
	virtual bool PerformNaturalSelection( TPOP*& rpCurrentPopulation );

	void UnCheckAllIndividuals();
	bool DecideToPerformMutation();
	bool DecideToPerformCrossOver();	
	TINDIV* GetRandomUncheckedPair( int nStartPoint );
	
	CTimeSpan	m_TimeSpan;

	int			m_nInitialSize;	
	double		m_fMutationRate;
	double		m_fCrossOverRate;
	int			m_nIterationCount;		
	TPOP*		m_pCurPop;
	TINDIV*		m_pHistoricalBest;
	DWORD_PTR	m_dwAppData;

	int			m_nLastRefinedIterationNo;	
};

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Function Name:	Constructor	
// Parameters	:	void
// Return Type	:	void
// Description	:	Constructor	
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
CGenerator<TPOP, TINDIV, TBIT>::CGenerator()
{
	m_nInitialSize			= DEFAULT_POPULATION_SIZE;
	m_fCrossOverRate		= DEFAULT_CROSS_OVER_RATE;
	m_fMutationRate			= DEFAULT_MUTATION_RATE;
	
	m_pCurPop				= NULL;
	m_dwAppData				= NULL;
	m_pHistoricalBest		= NULL;
	m_TimeSpan				= 0;
	
	m_nIterationCount			= 0;
	m_nLastRefinedIterationNo	= 0;	
}

//////////////////////////////////////////////////////////////////////
// Function Name:	Destructor	
// Parameters	:	void
// Return Type	:	void
// Description	:	Destructor	
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
CGenerator<TPOP, TINDIV, TBIT>::~CGenerator()
{
	Free();
}

////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::Free()
{
	if( NULL != m_pCurPop )	
	{		
		delete m_pCurPop;
		m_pCurPop = NULL;
	}

	if( NULL != m_pHistoricalBest )
	{
		delete m_pHistoricalBest;
		m_pHistoricalBest = NULL;
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	Destructor	
// Parameters	:	void
// Return Type	:	void
// Description	:	Destructor	
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::SetAppData( DWORD_PTR adwAppData )
{
	m_dwAppData = adwAppData;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	Destructor	
// Parameters	:	void
// Return Type	:	void
// Description	:	Destructor	
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
DWORD_PTR CGenerator<TPOP, TINDIV, TBIT>::GetAppData( )
{
	return m_dwAppData; 
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetTotalTimeSpan	
// Parameters	:	void
// Return Type	:	CTimeSpan
// Description	:	returns the total time as timespan	
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
CTimeSpan CGenerator<TPOP, TINDIV, TBIT>::GetTotalTimeSpan()
{
	return m_TimeSpan;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	CreatePopulation	
// Parameters	:	int nBitSize
// Return Type	:	void
// Description	:	Creates the population with m_nInitialSize as random
// Assumptions	:	Assumed that, this function will be called only once.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::CreatePopulation( int nIndividualCount, int nBitSize )
{
	ASSERT( nIndividualCount > 0 );
	//fatal error

	ASSERT( nBitSize > 0 );
	//fatal error

	ASSERT( NULL == m_pCurPop );  
	//if not, it means this method has been already called.
	
	CTime tStart = CTime::GetCurrentTime();

	m_nInitialSize = nIndividualCount;
	
	int nAny = 0;	
	srand( (unsigned)time( NULL ) );
	
	m_pCurPop = new TPOP();
	m_pCurPop->CreateIndividuals( m_nInitialSize );

	for( int nPersonX = 0; nPersonX<m_nInitialSize; nPersonX++ )
	{
		TINDIV* pPerson = new TINDIV();
		pPerson->CreateBits( nBitSize );

		for( int nBitX = 0; nBitX<nBitSize; nBitX++ )
		{
			TBIT* pBit = new TBIT();
			nAny = rand();
			if( 0 == ( nAny%2 ) )
			{
				pBit->SetValue( false );
			}
			else
			{
				pBit->SetValue( true );
			}

			pPerson->SetBit( nBitX, pBit );
		}

		double fPerf = pPerson->EvaluatePerformance( m_dwAppData );
		pPerson->SetPerformance( fPerf );
		m_pCurPop->SetIndividual( nPersonX, pPerson );
	}    
		
	m_pCurPop->Sort();
	RestoreHistoricalBest();
	//no need to update m_nLastRefinedIterationNo; this is first call.. iterationNo = 0;

	CTime tEnd = CTime::GetCurrentTime();
	m_TimeSpan = ( tEnd - tStart );
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetPopulationSize	
// Parameters	:	void
// Return Type	:	int nPopulationSize
// Description	:	Returns the size of the population. The population may
//					have been created or not! 
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
int CGenerator<TPOP, TINDIV, TBIT>::GetPopulationSize( )
{
	return m_nInitialSize;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	SetCrossOverRate
// Parameters	:	double fCrossOverRate ( default value is 0.60f )
// Return Type	:	void
// Description	:	see ProduceNextGeneration
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::SetCrossOverRate( double fCrossOverRate )
{
	ASSERT( fCrossOverRate >= 0.0f );
	//fatal error

	ASSERT( fCrossOverRate <= 1.0f );
	//fatal error	

	m_fCrossOverRate = fCrossOverRate;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetCrossOverRate
// Parameters	:	void
// Return Type	:	double fCrossOverRate
// Description	:	see SetCrossOverRate, ProduceNextGeneration
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
double CGenerator<TPOP, TINDIV, TBIT>::GetCrossOverRate( )
{
	return m_fCrossOverRate;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	SetMutationRate
// Parameters	:	double fMutationRate ( default value is 0.001f )
// Return Type	:	void
// Description	:	see ProduceNextGeneration
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::SetMutationRate( double fMutationRate )
{
	ASSERT( fMutationRate >= 0.0f );
	//fatal error

	ASSERT( fMutationRate <= 1.0f );
	//fatal error

	m_fMutationRate = fMutationRate;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetMutationRate
// Parameters	:	void
// Return Type	:	double fMutationRate
// Description	:	see SetMutationRate, ProduceNextGeneration
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
double CGenerator<TPOP, TINDIV, TBIT>::GetMutationRate( )
{
	return m_fMutationRate;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetCurrentBias
// Parameters	:	bool bReevaluate: True means, re-calculate the whole things 
//					and then return. False means, just return the last calculated value.
// Return Type	:	double fBias
// Description	:	see CPopulation::GetBias()
//					Bias means "How much do the individuals look like each other?"
// Assumptions	:	Assumed that, this function will be called after CreatePopulation.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
double CGenerator<TPOP, TINDIV, TBIT>::GetCurrentBias()
{
	ASSERT( NULL != m_pCurPop );
	//make sure that,  CreatePopulation has already been called.

	return m_pCurPop->CalculateBias();
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetAveragePerformance
// Parameters	:	bool bReevaluate: True means, re-calculate the whole things 
//					and then return. False means, just return the last calculated value.
// Return Type	:	double fAveragePerformance
// Description	:	see CPopulation::CalculateAvgPerf(),
//					see CIndividual::EvaluatePerformance(), 
// Assumptions	:	Assumed that, this function will be called after CreatePopulation.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
double CGenerator<TPOP, TINDIV, TBIT>::GetAveragePerformance( bool bReevaluate )
{
	ASSERT( NULL != m_pCurPop );
	//make sure that,  CreatePopulation has already been called.

	return m_pCurPop->CalculateAvgPerf( bReevaluate );
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetCurrentPopulation
// Parameters	:	void
// Return Type	:	TPOP*
// Description	:	Returns the current population. //may be NULL
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
TPOP* CGenerator<TPOP, TINDIV, TBIT>::GetCurrentPopulation()
{
	return m_pCurPop;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	PerformNaturalSelection( virtual function: have to be overwritten )
// Parameters	:	TPOP*& rpCurrentPopulation
// Return Type	:	bool "IsPerformSort" 
// Description	:	Performs the natural selection.
// Assumptions	:	Should be called only in ProduceNextGeneration()
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
bool CGenerator<TPOP, TINDIV, TBIT>::PerformNaturalSelection( TPOP*& rpCurrentPopulation )
{
	ASSERT( FALSE );
	return false;
	//Please OVERWRITE or EDIT this function
	//Otherwise it will always assert.

	//CHOOSE one of the followings with your parameters or
	//implement your selection.

	//double fAvgPerf = rpCurrentPopulation->CalculateAvgPerf();
	//rpCurrentPopulation->RouletteWheelProportionate( fAvgPerf );
	//return false;
	
	//double fAvgPerf = rpCurrentPopulation->CalculateAvgPerf();
	//rpCurrentPopulation->StochasticRemainderSelection( fAvgPerf );
	//return false;

	//rpCurrentPopulation->GenitorSelection( 3 );
	//return true;

	//rpCurrentPopulation->TournamentSelection( 0.75f );
	//return false;

	//customSelection?
	//return (you need sort w.r.t performance?)
	//return (you need a ranking for the next iteration?)
	//if so please return true; [don't worry for first call, it it sorted.]
	//else return false to increase the speed of the calculation.
	//If you need a sort, DON'T DIRECTLY CALL "Sort()";	
}

//////////////////////////////////////////////////////////////////////
// Function Name:	DecideToPerformCrossOver
// Parameters	:	void
// Return Type	:	bool
// Description	:	returns the decision about perform(don't perform) crossOver
// Assumptions	:	Should be called only in ProduceNextGeneration()
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
bool CGenerator<TPOP, TINDIV, TBIT>::DecideToPerformCrossOver()
{
	if( ( m_fCrossOverRate * RANDOM_INT_PRECISION_ER ) > ( rand() % RANDOM_INT_PRECISION_ER ) )
	{
		return true;
	}
	else
	{
		return false;
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	DecideToPerformMutation
// Parameters	:	void
// Return Type	:	bool
// Description	:	returns the decision about perform(don't perform) mutation
// Assumptions	:	Should be called only in ProduceNextGeneration()
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
bool CGenerator<TPOP, TINDIV, TBIT>::DecideToPerformMutation()
{
	if( ( m_fMutationRate* RANDOM_INT_PRECISION_ER ) > ( rand() % RANDOM_INT_PRECISION_ER ) )	
	{
		return true;
	}
	else
	{
		return false;
	}
}


//////////////////////////////////////////////////////////////////////
// Function Name:	PerformCrossOver( virtual function: have to be overwritten )
// Parameters	:	TINDIV*& rpPerson	: one of the parent to be crossedOver	
//					TINDIV*& rpPair	: other parent to be crossedOver	
// Return Type	:	void
// Description	:	Performs the natural selection.
// Assumptions	:	Should be called only in ProduceNextGeneration()
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::PerformCrossOver( TINDIV*& rpPerson, TINDIV*& rpPair )
{
	ASSERT( FALSE ); 
	//Please OVERWRITE or EDIT this function
	//Otherwise it will always assert.

	//CHOOSE one of the followings with your parameters 

	//////////// ONE POINT CROSS-OVER /////////////////////////////////////////
	//int nStartIndex = rand() % rpPerson->GetBitSize(); //or set another index 
	//rpPerson->OnePointCrossOver( rpPair, nStartIndex );
	
	//////////// TWO POINT CROSS-OVER /////////////////////////////////////////
	//int nStartIndex = rand() % rpPerson->GetBitSize(); //or set another index 
	//int nEndIndex = nStartIndex + ( rand() % ( rpPerson->GetBitSize() - nStartIndex ) );
	//rpPerson->TwoPointCrossOver( rpPair, nStartIndex, nEndIndex );

	//////////// UNIFORM CROSS-OVER /////////////////////////////////////////
	//rpPerson->UniformCrossOver( rpPair, 0.40 );
}

//////////////////////////////////////////////////////////////////////
// Function Name:	PerformMutation
// Parameters	:	TINDIV*& rpPerson	: the parent to be mutated	
// Return Type	:	void
// Description	:	Performs the mutation.
// Assumptions	:	Should be called only in ProduceNextGeneration()
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::PerformMutation( TINDIV*& rpPerson )
{
	int nMutationIndex = rand() % rpPerson->GetBitSize();
	rpPerson->Mutate( nMutationIndex );
}

//////////////////////////////////////////////////////////////////////
// Function Name:	ProduceNextGeneration
// Parameters	:	void
// Return Type	:	void
// Description	:	Performs the main functionality of the class
// Assumptions	:	Assumed that, "CreatePopulation" have been
//					already called.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::ProduceNextGeneration()
{
	//make sure that the population has been already cretaed
	//i.e. CreatePopulation has been called
	ASSERT( NULL != m_pCurPop );
	
	//to calculate the time consuming...
	CTime tStart = CTime::GetCurrentTime();

	//seed the randomizer
	srand( (unsigned)time( NULL ) );
	
	//natural selection...
	//After this call, m_pCurPop has been refreshed.
	//It means, any derived results like performances, 
	//average preformances, bias etc. were destroyed.
	//This condition will remain until, the following
	//functions are called: CalculateAvgPerformance(true), Sort()
	
	bool bSort = PerformNaturalSelection( m_pCurPop );
	
	//iterate...
	int nIndex = 0;
	bool bContinue = true;
	while( nIndex<m_nInitialSize && bContinue )	
	{
		//get an individual in order
		TINDIV* pPerson = m_pCurPop->GetIndividual( nIndex );
		//check whether it has tracked in this iteration or not
		if( !pPerson->IsChecked() )
		{
			//get an unTracked pair for it, searching the rest of
			//the population... It should be selected randomly!
			TINDIV* pPair = GetRandomUncheckedPair( nIndex );
			//if return is NULL, it means all the population 
			//is tracked in this iteration
			if( NULL != pPair ) //we have successfully get a pair.
			{  
				if( DecideToPerformCrossOver() )
				{	
					//perform crossover
					PerformCrossOver( pPerson, pPair );
					//flag that those individuals have been
					//tracked in this iteration...
					pPerson->SetChecked( true );
					pPair->SetChecked( true );
				}				
			}			
			else
			{
				//we couldn't find a pair. It means all the 
				//population is tracked in this iteration.
				//Stop
				bContinue = false;
			}
		}
		
		//mutation		
		if( DecideToPerformMutation() )
		{
			PerformMutation( pPerson );
		}

		//evaluate and set last performances...
		double tempPer = pPerson->EvaluatePerformance( m_dwAppData );
		pPerson->SetPerformance( tempPer );

		nIndex++;
	}
	
	//uncheck for 
	UnCheckAllIndividuals();	
	
	//is sorting necessary for next iteration
	if( bSort )
	{
		m_pCurPop->Sort();
	}
	
	//restore the king :)
	bool bRefined = RestoreHistoricalBest();
	
	//calculate the time consuming...
	CTime tEnd = CTime::GetCurrentTime();
	//add it to the totalTime...
	m_TimeSpan += ( tEnd - tStart );

	m_nIterationCount++;	

	if( bRefined )
	{		
		m_nLastRefinedIterationNo = m_nIterationCount;
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	UnCheckAllIndividuals
// Parameters	:	void
// Return Type	:	void
// Description	:	unchecks all individuals
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
void CGenerator<TPOP, TINDIV, TBIT>::UnCheckAllIndividuals()
{
	for( int x=0; x<m_nInitialSize; x++ )	
	{
		TINDIV* pPerson = m_pCurPop->GetIndividual( x );
		pPerson->SetChecked( false );
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetRandomUncheckedPair
// Parameters	:	int nStartPoint, the pair will search after this index.
// Return Type	:	TINDIV*
// Description	:	The pair will slect the rest of the 
//					population by searching ( randomly) the array after 
//					the index "startPoint". The Checked Ones will be omitted.
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
TINDIV* CGenerator<TPOP, TINDIV, TBIT>::GetRandomUncheckedPair( int nStartPoint )
{	
	ASSERT( NULL != m_pCurPop );
	//fatal error
	
	ASSERT( nStartPoint < m_nInitialSize );
	//fatal error

	int nPairIndex = rand() % ( m_nInitialSize - nStartPoint );
	if( 0 == nPairIndex )
	{
		nPairIndex++;
	}
	
	for( int x=nStartPoint+nPairIndex; x<m_nInitialSize; x++ )
	{
		TINDIV* pTemp = m_pCurPop->GetIndividual( x );
		if( ! pTemp->IsChecked() )
		{
			return pTemp;
		}
	}

	return NULL;	
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetIterationCount
// Parameters	:	void
// Return Type	:	int
// Description	:	returns the iteration Count
// Assumptions	:	No Assumption
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
int CGenerator<TPOP, TINDIV, TBIT>::GetIterationCount()
{
	return m_nIterationCount;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	RestoreHistoricalBest
// Parameters	:	
// Return Type	:	bool describes whether it is restored or no need to restoration.
// Description	:	Restores the historical best individual...
// Assumptions	:	Assumed that, this function will be called after CreatePopulation.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
bool CGenerator<TPOP, TINDIV, TBIT>::RestoreHistoricalBest( )
{
	TINDIV* pCurBest  = GetFittest();

	if( NULL == m_pHistoricalBest )	
	{		
		m_pHistoricalBest = pCurBest->Clone();
		return true;
	}
	else
	{		
		if( pCurBest->GetPerformance() > m_pHistoricalBest->GetPerformance() )
		{
			delete m_pHistoricalBest;
			m_pHistoricalBest = pCurBest->Clone();
			return true;
		}
	}

	return false;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetHistoricalBest
// Parameters	:	
// Return Type	:	TINDIV*
// Description	:	Returns the individual which has the best performance during alliterations.
// Assumptions	:	Assumed that, this function will be called after CreatePopulation.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
TINDIV* CGenerator<TPOP, TINDIV, TBIT>::GetHistoricalBest( )
{
	return m_pHistoricalBest;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetFittest
// Parameters	:	
// Return Type	:	TINDIV*
// Description	:	Returns the best individual
// Assumptions	:	Assumed that, this function will be called after CreatePopulation.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
TINDIV* CGenerator<TPOP, TINDIV, TBIT>::GetFittest( )
{
	ASSERT( NULL != m_pCurPop );
	//fatal error
	
	double fBest	   = -1000000000000.0f;
	double fCurrent = 0.0f; 
	TINDIV* pBest = NULL;

	for( int x=0; x<m_nInitialSize; x++  )
	{
		TINDIV* pPerson = m_pCurPop->GetIndividual( x );
		
		fCurrent = pPerson->GetPerformance() ;

		if( fBest < fCurrent )
		{
			fBest = fCurrent;
			pBest = pPerson;
		}
	}

	return pBest;
}


//////////////////////////////////////////////////////////////////////
// Function Name:	GetFittestPerformance
// Parameters	:	bool bReevaluate: True means, re-calculate the whole things 
//					and then return. False means, just return the last calculated value.
// Return Type	:	double theFittestPerformance
// Description	:	Returns the performance of the best individual
// Assumptions	:	Assumed that, this function will be called after CreatePopulation.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
double CGenerator<TPOP, TINDIV, TBIT>::GetFittestPerformance( bool bReevaluate )
{
	ASSERT( NULL != m_pCurPop );
	//fatal error
	
	double fBest	   = 0.0f;
	double fCurrent = 0.0f; 
	for( int x=0; x<m_nInitialSize; x++  )
	{
		TINDIV* pPerson = m_pCurPop->GetIndividual( x );
		if( bReevaluate )
		{
			fCurrent = pPerson->EvaluatePerformance();
			pPerson->SetPerformance( fCurrent );
		}
		else
			fCurrent = pPerson->GetPerformance() ;

		if( fBest < fCurrent )
		{
			fBest = fCurrent;
		}
	}

	return fBest;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetLastRefinedIterationNo
// Parameters	:	void
// Return Type	:	int LastRefinedIterationNo
// Description	:	Returns the iteration number of last improvement process.
// Assumptions	:	Assumed that, this function will be called after ProduceNextGeneration.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
int CGenerator<TPOP, TINDIV, TBIT>::GetLastRefinedIterationNo()
{
	return m_nLastRefinedIterationNo;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetSuccessiveUnRefinedIterationCount
// Parameters	:	void
// Return Type	:	int 'successive UnRefined Iteration Count'
// Description	:	Returns the last value of iteration count which has been spent waste.
// Assumptions	:	Assumed that, this function will be called after ProduceNextGeneration.
////////////////////////////////////////////////
template< class TPOP, class TINDIV, class TBIT >
////////////////////////////////////////////////
int CGenerator<TPOP, TINDIV, TBIT>::GetSuccessiveUnRefinedIterationCount()
{
	return GetIterationCount() - GetLastRefinedIterationNo();
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
#endif // !defined(__GENERATOR_H__)
