//+------------------------------------------------------------------------
//
//  Copyright @ Baris TAZE
//
//  File:     Individual.h
//
//  Content:  Interface and Implementation for CIndividual class
//  
//  Classes:  CIndividual
//            
//  History:  27-May-2002    created by baris taze .  
//
//-------------------------------------------------------------------------

#if !defined(__INDIVIDUAL_H__)
#define      __INDIVIDUAL_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
class CIndividual  
{
public:
	CIndividual( );
	~CIndividual();

	void Free();

	virtual TINDIV* Clone();

	void CreateBits( int nBitSize );		
	virtual bool IsEqual( TINDIV* pRHS );
		
	void	SetBit ( int nIndex, TBIT* pBit );		
	TBIT*	GetBit( int nIndex );	

	int		GetBitSize();
	
	virtual double EvaluatePerformance( DWORD_PTR adwAppData );	
	void	SetPerformance( double fPerformance );
	double	GetPerformance( );
	
	void Mutate( int nIndex );
	
	//crossover functions
public:
	void OnePointCrossOver( TINDIV* pPair, int nStartBit );
	void TwoPointCrossOver( TINDIV* pPair, int nStartBit, int nEndBit );
	void UniformCrossOver ( TINDIV* pPair, double afSwapRate );

protected:
	void PointCrossOver( TINDIV* pPair, int nStartBit, int nLength );
		
public:
	void Lock( int nStartIndex, int nLength );
	void UnLock( int nStartIndex, int nLength );
	bool IsLocked( int nIndex );
	
	void SetChecked( bool bChecked );
	bool IsChecked();
	
	long GetDecimal( int nStartIndex, int nLength );

	CString ToString();

protected:
	TBIT** m_pBits;	 
	int	   m_nBitSize;   
	double  m_fPerformance;	
	bool   m_bChecked;
};

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Function Name:	Constructor
// Parameters	:	void
// Return Type	:	void
// Description	:	Constructor	
// Assumptions	:	No Assumption
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
CIndividual<TINDIV, TBIT>::CIndividual()
{
	m_pBits			= NULL;
	m_nBitSize		= 0;
	m_fPerformance	= 0.0f;
	m_bChecked		= false;	
}

//////////////////////////////////////////////////////////////////////
// Function Name:	Clone( virtual function: have to be overwrite )
// Parameters	:	void
// Return Type	:	TINDIV*
// Description	:	Copys this object and return
// Assumptions	:	No Assumption
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
TINDIV* CIndividual<TINDIV, TBIT>::Clone()
{
	TINDIV* pTemp = new TINDIV();

	if( NULL != m_pBits )
	{
		pTemp->CreateBits( m_nBitSize );
		for( int x=0; x<m_nBitSize; x++ )
		{
			TBIT* pBit = new TBIT();
			*pBit = *m_pBits[ x ];
			pTemp->SetBit( x, pBit );
		}

		pTemp->SetPerformance( m_fPerformance );
		pTemp->SetChecked( m_bChecked );
	}

	return pTemp;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	CreateBits
// Parameters	:	int nBitSize
// Return Type	:	void
// Description	:	Creates the bit string of this individuals
// Assumptions	:	Assumed that this function will be called only once.
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::CreateBits( int nBitSize )
{		
	ASSERT( nBitSize > 0 );
	//fatal error

	ASSERT( NULL == m_pBits );
	//call this function only once.

	m_nBitSize = nBitSize;
	m_pBits = new TBIT*[ m_nBitSize ];
	for( int x=0; x<m_nBitSize; x++ )
	{
		m_pBits[ x ] = NULL;
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	Destructors
// Parameters	:	void
// Return Type	:	void
// Description	:	Destructors	
// Assumptions	:	No Assumption
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
CIndividual<TINDIV, TBIT>::~CIndividual()
{
	Free();
}

////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::Free()
{
	if( NULL != m_pBits )
	{
		for( int x=0; x<m_nBitSize; x++ )
		{
			if( NULL != m_pBits[ x ] )
			{
				delete m_pBits[ x ];
				m_pBits[ x ] = NULL;
			}
		}

		delete[] m_pBits;
		m_pBits = NULL;
	}

	m_nBitSize = 0;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	IsEqual ( virtual function: have to be override )	
// Parameters	:	TINDIV* pRHS, the object to be checked for equality
// Return Type	:	bool
// Description	:	checks whether to the bit string values of each individual is equal or not.
// Assumptions	:	No Assumption
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
bool CIndividual<TINDIV, TBIT>::IsEqual( TINDIV* pRHS )
{
	ASSERT( NULL != pRHS );
	ASSERT( m_nBitSize == pRHS->GetBitSize() );

	for( int x=0; x<m_nBitSize; x++ )
	{
		if( !GetBit( x )->IsEqual( pRHS->GetBit( x ) ) )
		{
			return false;
		}		
	}

	return true;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	SetBit
// Parameters	:	int nIndex, position to be set.
//					TBIT* pBit, the object to be set.
// Return Type	:	void
// Description	:	sets the bit to the desired location.
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 					
//					Assumed that, the old object is freed explicitly or 
//					it is still on use. [ the old one have to be explicitly deleted ]
//					It may be NULL, too.
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::SetBit ( int nIndex, TBIT* pBit )
{
	ASSERT( nIndex >= 0 );
	//fatal error

	ASSERT( nIndex < m_nBitSize );
	//fatal error

	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	m_pBits[ nIndex ] = pBit;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetBit
// Parameters	:	int nIndex, position to be set.
//					TBIT* pBit, the object to be set.
// Return Type	:	TBIT* //may be NULL
// Description	:	sets the bit to the desired location.
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
TBIT* CIndividual<TINDIV, TBIT>::GetBit( int nIndex )
{
	ASSERT( nIndex >= 0 );
	//fatal error

	ASSERT( nIndex < m_nBitSize );
	//fatal error

	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	return m_pBits[ nIndex ];	
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetBitSize
// Parameters	:	void
// Return Type	:	int
// Description	:	returns the bitSize of this individual.
// Assumptions	:	No Assumption
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
int	CIndividual<TINDIV, TBIT>::GetBitSize()
{
	return m_nBitSize;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetPerformance
// Parameters	:	void
// Return Type	:	double
// Description	:	returns the performance of this individual.
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
double CIndividual<TINDIV, TBIT>::GetPerformance( )
{
	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	return m_fPerformance;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	SetPerformance
// Parameters	:	double fPerformance
// Return Type	:	void
// Description	:	sets the performance of this individual.
// Assumptions	:	No Assumption
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::SetPerformance( double fPerformance )
{
	m_fPerformance = fPerformance;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	EvaluatePerformance ( virtual func: have to be overwriten )
// Parameters	:	void
// Return Type	:	double
// Description	:	returns the performance of this individual.
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
double CIndividual<TINDIV, TBIT>::EvaluatePerformance( DWORD_PTR adwAppData )
{
	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	ASSERT( false );
	//overwrite or edit this function.
	//Otherwise it always asserts.

	return 0.0f;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	Mutate
// Parameters	:	int nIndex, the location of the bit to be mutated.
// Return Type	:	void
// Description	:	swap the deired bit [ from 1 to 0, or 0 to 1 ]
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::Mutate( int nIndex )
{	
	ASSERT( nIndex >= 0 );
	//fatal error

	ASSERT( nIndex < m_nBitSize );
	//fatal error
	
	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	if( ! m_pBits[ nIndex ]->IsLocked() )
	{
		m_pBits[ nIndex ]->Mutate();
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	OnePointCrossOver
// Parameters	:	TINDIV* pPair, to be crossed over.
//					int nStartBit, the location of the startBit(included)
// Return Type	:	void
// Description	:	exchange the subpart of the bit string 
//					between two bits correspondingly.
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::OnePointCrossOver( TINDIV* pPair, int nStartBit )
{
	ASSERT( nStartBit >= 0 );
	//fatal error

	ASSERT( nStartBit < m_nBitSize );
	//fatal error

	ASSERT( NULL != pPair );
	//fatal error
	
	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	//optimize the for loop in PointCrossOver
	if( nStartBit < ( m_nBitSize - nStartBit ) )   //xxxx|xxxxxxxxxxx : swap left part
	{
		PointCrossOver( pPair, 0, nStartBit + 1 ); 
	}
	else  //xxxxxxxxx|xxx : swap right part
	{
		PointCrossOver( pPair, nStartBit, m_nBitSize - nStartBit ); 	
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	TwoPointCrossOver
// Parameters	:	TINDIV* pPair, to be crossed over.
//					int nStartBit, the location of the startBit(included)
//					int nEndBit,  the location of the endBit(included)
// Return Type	:	void
// Description	:	exchange the subpart of the bit string 
//					between two bits correspondingly.
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::TwoPointCrossOver( TINDIV* pPair, int nStartBit, int nEndBit )
{
	ASSERT( nStartBit >= 0 );
	//fatal error

	ASSERT( nEndBit >= 0 );
	//fatal error

	ASSERT( nStartBit < m_nBitSize );
	//fatal error

	ASSERT( nEndBit < m_nBitSize );
	//fatal error

	ASSERT( nStartBit <= nEndBit );
	//fatal error

	ASSERT( NULL != pPair );
	//fatal error
	
	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.


	//optimize the for loop in PointCrossOver
	if( ( nEndBit - nStartBit ) > ( m_nBitSize / 2 ) ) // xx|xxxxxxxxxxxxxxx|xxx
	{
		OnePointCrossOver( pPair, nStartBit );
		OnePointCrossOver( pPair, nEndBit );
	}
	else// xxxxxxxxxxxxxxx|xxxxx|xxxxxxx
	{
		PointCrossOver( pPair, nStartBit, ( nEndBit - nStartBit + 1 ) ); 
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	UniformCrossOver
// Parameters	:	TINDIV* pPair, to be crossed over.
// Return Type	:	void
// Description	:	bit positions:    1    2    3    4    5
//					---------------------------------------	
//					first   parent:   0    0    0    1    1 
//					second  parent:   1    1    1    0    0
//					c.over pattern: swap  same swap same swap (randomly)
//					---------------------------------------	
//					first    child:   1    0    1    1    0 
//					second   child:   0    1    0    0    1							  
//					---------------------------------------	
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::UniformCrossOver( TINDIV* pPair, double afSwapRate/*btw 0 and 1*/ )
{
	ASSERT( NULL != pPair );
	//fatal error

	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.
	
	TBIT* pTemp;
	int nSwap = (int)((double)1000 * afSwapRate);
	for( int x=0; x<m_nBitSize; x++ )
	{
		int nRandom = rand() % 1000;
		if( nRandom < nSwap  ) 
		{
			pTemp = m_pBits[ x ]; 
			m_pBits[ x ] = pPair->GetBit( x );
			pPair->SetBit( x, pTemp );		
		}
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	TwoPointCrossOver
// Parameters	:	TINDIV* pPair, to be crossed over.
//					int nStartBit, the location of the startBit(included)
//					int nLength, of the area to be crossover. 
// Return Type	:	void
// Description	:	exchange the subpart of the bit string 
//					between two individuals correspondingly.
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::PointCrossOver( TINDIV* pPair, int nStartBit, int nLength )
{	
	ASSERT( NULL != pPair );
	//fatal error

	ASSERT( nStartBit >= 0 );
	//fatal error

	ASSERT( nStartBit+nLength <= m_nBitSize );
	//fatal error
	
	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	TBIT* pTemp;
	for( int x=0; x<nLength; x++ )
	{
		pTemp = m_pBits[ nStartBit + x ]; 
		m_pBits[ nStartBit + x ] = pPair->GetBit( nStartBit + x );
		pPair->SetBit( nStartBit + x, pTemp );
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	Lock
// Parameters	:	int nStartIndex, int nLength
// Return Type	:	void
// Description	:	Sets the "lock state" of the "nLength" bits as true, 
//					starting from "nStartIndex".
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::Lock( int nStartIndex, int nLength )
{
	ASSERT( nStartIndex >= 0 );
	//fatal error

	ASSERT( nStartIndex+nLength < m_nBitSize );
	//fatal error

	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	for( int x=0; x<nLength; x++ )
	{
		m_pBits[ nStartIndex + x ]->Lock();
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	UnLock
// Parameters	:	int nStartIndex, int nLength
// Return Type	:	void
// Description	:	Sets the "lock state" of the "nLength" bits as false, 
//					starting from "nStartIndex".
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::UnLock( int nStartIndex, int nLength )
{
	ASSERT( nStartIndex >= 0 );
	//fatal error

	ASSERT( nStartIndex+nLength < m_nBitSize );
	//fatal error

	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	for( int x=0; x<nLength; x++ )
	{
		m_pBits[ nStartIndex + x ]->Unlock();
	}
}

//////////////////////////////////////////////////////////////////////
// Function Name:	IsLocked
// Parameters	:	int nIndex
// Return Type	:	bool 
// Description	:	Returns the "lock state" of the bit located in "nIndex".
// Assumptions	:	Assumed that, "CreateBits()" has been already called. 
//					Assumed that, all bits are created explicitly and set.	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
bool CIndividual<TINDIV, TBIT>::IsLocked( int nIndex )
{
	ASSERT( nIndex >= 0 );
	//fatal error

	ASSERT( nIndex < m_nBitSize );
	//fatal error

	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	return m_pBits[ nIndex ]->IsLocked();
}

//////////////////////////////////////////////////////////////////////
// Function Name:	ToString
// Parameters	:	void
// Return Type	:	CString
// Description	:	Returns the bits like "11011001010".
// Assumptions	:	No Assumption	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
CString CIndividual<TINDIV, TBIT>::ToString()
{
	CString temp;
	for( int x=0; x<m_nBitSize; x++ )
	{
		temp += m_pBits[x]->ToString();
	}
	return temp;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	SetChecked
// Parameters	:	void
// Return Type	:	bool
// Description	:	Sets the check status of this individual
// Assumptions	:	No Assumption	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
void CIndividual<TINDIV, TBIT>::SetChecked( bool bChecked )
{
	m_bChecked = bChecked;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	IsChecked
// Parameters	:	bool
// Return Type	:	void
// Description	:	Returns the check status of this individual
// Assumptions	:	No Assumption	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
bool CIndividual<TINDIV, TBIT>::IsChecked()
{
	return m_bChecked;
}

//////////////////////////////////////////////////////////////////////
// Function Name:	GetDecimal
// Parameters	:	int nStartIndex, int nLength
// Return Type	:	long
// Description	:	Converts the referenced binary part to a numeric value
// Assumptions	:	Assumed that, the length doesn't produce an "overflow"!	
////////////////////////////////////
template< class TINDIV, class TBIT >
////////////////////////////////////
long CIndividual<TINDIV, TBIT>::GetDecimal( int nStartIndex, int nLength )
{
	//left to right   xxxxx100xx = 4, start index = 5, length = 3;

	ASSERT( nStartIndex >= 0 );
	//fatal error

	ASSERT( nStartIndex+nLength <= m_nBitSize );
	//fatal error

	ASSERT( NULL != m_pBits );
	//make sure that CreateBits() has been already called.

	long data = 0L;

	for( int x=0; x<nLength; x++ )	
	{
		TBIT* pBit = GetBit( nStartIndex + x );
		data *= 2;
		if( pBit->GetValue() ) //1
		{
			data++;
		}		
	}

	return data;
}

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

#endif // !defined(__INDIVIDUAL_H__)
